library(plm)
temp1=read.csv("productivity.csv")
myData1= plm.data(temp1, index = c("state", "year"))
head(myData1)


Lhs= names(myData1)[3]
Rhs= paste(names(myData1)[4:9],collapse="+")
myEq=paste(Lhs, Rhs, sep="~")
myFormula=as.formula(myEq)

gsp_pool = plm(myFormula, data = myData1, model = "pooling")
summary(gsp_pool)

gsp_fe=plm(myFormula, data = myData1, model = "within")
summary(gsp_fe)

gsp_re = plm(myFormula, data = myData1, model = "random", random.method = "swar")
summary(gsp_re)



###################################################
### Chapter 4. More Topics: two-way 
###################################################
gsp_fe_t = plm(myFormula,effect = "time", model = "within", data = myData1)
summary(gsp_fe_t)
gsp_fe_2 = plm(myFormula, effect ="twoways", model = "within", data = myData1)
summary(gsp_fe_t)

FE=fixef(gsp_fe_2, effect= "individual")
PE=fixef(gsp_fe_2, effect= "time")
as.matrix(FE)
PEt=ts(PE,start=1970,freq=1)
plot(PEt,ylab="Period effects")

gsp_re_2=plm(myFormula, effect ="twoways", model = "random", data = myData1)
summary(gsp_re_2)$coef

gsp_re2ML<-lme(myFormula,data=temp1,random=~1|state/year)
summary(gsp_re2ML)$tTable
summary(gsp_re2ML)$coef$random$state
m=t(matrix(summary(gsp_re2ML)$coef$random$year,48,))
apply(m,1,mean)


###  Variable Coefficients Models
tempx = read.csv("retVol.csv")
DataVC= plm.data(tempx, index = c("ID","DATEID"))
head(DataVC)
pdim(DataVC)
myFormVC=RET ~ STD + log(VOLUMN)
pvcm_FE = pvcm(myFormVC, data = DataVC)
summary(pvcm_FE)
pvcm_RE = pvcm(myFormVC, data = DataVC, model = "random")
summary(pvcm_RE)


summary(pvcm_FE)$coef[,2:3]


###################################################
### Chapter 5. tests
###################################################
pFtest(gsp_fe, gsp_pool)
pooltest(gsp_pool,gsp_fe)


plmtest(gsp_pool,effect="individual",type="honda")
plmtest(gsp_pool,effect="individual",type="bp")
plmtest(gsp_pool,effect="twoways",type="ghm")
plmtest(gsp_pool,effect="twoways",type="kw")
#effect = c("individual", "time", "twoways"),
#type = c("honda","bp", "ghm", "kw")


### Hausman Test
phtest(gsp_re, gsp_fe)


### Testing for Serial Correlation
pwtest(gsp_pool)
pdwtest(gsp_re)
pbsytest(gsp_re, alternative = c("twosided","onesided"))
library(car)
pwartest(gsp_fe) # Using linearHypothesis() in package car



### Estimation accounting for Serial Correlation
gsp_feAR1=gls(myFormula,data=temp1, correlation=corAR1(0,form=~year|state))
summary(gsp_feAR1)


gsp_reAR1=lme(myFormula,data=temp1,random=~1|state,correlation = corAR1(0,form=~year|state))
summary(gsp_reAR1)

##### 

myFormula1=as.formula(paste(paste(myEq,"-1"),"as.factor(state)",sep="+"))

gsp_fe1m=gls(myFormula1,data=temp1)
names(summary(gsp_fe1m))
summary(gsp_fe1m)$BIC
para=summary(gsp_fe1m)$tTable
para[1:length(gsp_fe$coef),]

summary(gsp_fe)$coef

anova(gsp_fe1m,gsp_feAR1)

