library(plm)
temp1=read.csv("productivity.csv")
myData1= plm.data(temp1, index = c("state", "year"))
head(myData1);pdim(myData1)

Lhs= names(myData1)[3]
Rhs= paste(names(myData1)[4:9],collapse="+")
myEq=paste(Lhs, Rhs, sep="~")
myFormula=as.formula(myEq)

### Section 1: Estimation accounting for Serial Correlation
## FE AR(1)

gsp_feAR1=gls(myFormula,data=temp1, correlation=corAR1(0,form=~year|state))
summary(gsp_feAR1)
summary(gsp_feAR1)$tTable

## RE AR(1) by MLE
gsp_reAR1=lme(myFormula,data=temp1,random=~1|state,correlation = corAR1(0,form=~year|state))
summary(gsp_reAR1)
summary(gsp_reAR1)$tTable

##### ANOVA Comparing RE AR(1) vs FE AR(1)
anova(gsp_reAR1,gsp_feAR1)

##### ANOVA Comparing FE iid vs FE AR(1)
myFormula1=as.formula(paste(paste(myEq,"-1"),"as.factor(state)",sep="+"))

gsp_feIID=gls(myFormula1,data=temp1)
k=length(names(myData1)[4:9])
summary(gsp_feIID)$tTable[1:k,]
anova(gsp_feIID,gsp_feAR1)


### Section 2: Robust covariance
library(sandwich)
library(lmtest)
gsp_re = plm(myFormula, data = myData1, model = "random")

coeftest(gsp_re)
coeftest(gsp_re, vcov= vcovHC)
coeftest(gsp_re, vcov= vcovBK)
coeftest(gsp_re, vcov= vcovSCC)

coeftest(gsp_re, vcov= vcovHC(gsp_re, type="HC2", cluster="time", method="white2"))
coeftest(gsp_re, vcov= vcovBK(gsp_re, type="HC2", cluster="time"))
coeftest(gsp_re, vcov= vcovSCC(gsp_re, type="HC2"))

### Petersen(2008)
source("petersen.src")
gsp_lm = lm(myFormula, data = myData1)
cl(myData1,gsp_lm,state)			 # Clustered by firm
cl(myData1,gsp_lm, year)			 # Clustered by year
mcl(myData1,gsp_lm, state, year)		 # Clustered by firm and year






