###################################################
### filter1 eval=FALSE
###################################################
temp=read.csv("Ypk.csv", header=T)
head(temp)
vname=c("per capita GDP")
incomePK=ts(log(temp[,2]), start=c(1961,1),freq=4)
head(incomePK)
###################################################
### decompose
###################################################
y=window(incomePK,start=c(1990,1))
dd_dec=decompose(y)
par(mfrow=c(2,2))
plot(y,main="CHұo", ylab="")
plot(dd_dec$trend,main="Ͷզ", ylab="")
plot(dd_dec$seasonal,main="u`", ylab="")
plot(dd_dec$random,main="Hi", ylab="")
par(mfrow=c(1,1))

dd_stl = stl(y, s.window = 13)

plot(dd_stl)

###################################################
### Time serie Object Creation
###################################################

temp=read.csv("returns47.csv", header=TRUE)
names(temp)
head(temp)
ret47=ts(temp, start=c(1995,1),freq=12)
head(ret47)
save(ret47, file="ret47.RData")	
load("ret47.RData")
head(ret47)
class(ret47)
names(ret47)
colnames(ret47)
head(ret47)	
tail(ret47)

# Read daily data, the first column is date
library(timeSeries)  #library(xts)
temp=read.csv("indexReturns.csv", header=TRUE)
indexRet=as.timeSeries(temp)
head(indexRet)


###################################################
### Time serie Plot
###################################################
Y= indexRet[,5] 
plot(Y, type="l", col="blue", ylab="%", lwd=2, main="Index returns")
grid()
         +
         
plot(indexRet[, 5:8],main="") 
plot(indexRet[, 1:8],main="")

###################################################
### Descriptive statistics about y
###################################################
	## Summary statistics


mean(y)
sd(y)
summary(y)
plot(density(y), col="blue", main="(Unconditional) Density of inflation rates")


	## Histogram Plot
y=as.timeSeries(read.csv("tw_retVol.csv"))[,1]
hist(y, col = 3, probability = TRUE,main = "Market index returns",ylim=c(0,0.45))
curve(dnorm(x), min(y), max(y), add = TRUE, lwd = 2)


	## Distribution Plot
par(mfrow=c(2,2))

plot(density(y), main = "Kernel Density Estimate")
qqnorm(y, pch =19)
qqnorm(y, pch =19, main = "Normal Q-Q with qqline");qqline(y)
boxplot(y, col = "green", main = "Box-Plot")

par(mfrow=c(1,1))


	## ACf and PACF
temp=read.csv("tw_inf.csv")
inf=ts(temp[,2],start=c(1982,1),freq=12)
par(mfrow=c(2,1))
acf(inf, main="ACF of inflation rates")
pacf(inf,main="PACF of inflation rates")
par(mfrow=c(1,1))



###################################################
### Filter
###################################################
temp=read.csv("tw_inf.csv")
head(temp)
inf=ts(temp[,2], start=c(1982,1),freq=12)

plot(inf, ylab="", main="xWqfȲv, %")
f= filter(inf, c(1/2, rep(1, 11), 1/2)/12)
lines(f, col = 2)

###################################################
### rollapply
###################################################
library(zoo)
rlp_mean=rollapply(inf, 12, mean)
rlp_sd=rollapply(inf, 12, sd)

par(mfrow=c(2,1))
plot.ts(rlp_mean,main="Rolling average", ylab="")
plot.ts(rlp_sd,main="Rolling standard deviation", ylab="")
par(mfrow=c(1,1))
###################################################
### decompose
###################################################
dd_dec=decompose(inf)
dd_stl = stl(inf, s.window = 13)
plot(dd_dec)
plot(dd_stl)

###################################################
### decompose-components/seat-mean-sd
###################################################
plot(dd_dec$trend, ylab = "trend")
lines(dd_stl$time.series[,"trend"], lty = 2, lwd = 2)
plot(rollapply(inf, 12, sd))


###################################################
### Holt-Winters
###################################################
dd_past = window(inf, end = c(2011, 12))
dd_hw = HoltWinters(dd_past)
dd_pred = predict(dd_hw, n.ahead = 30)
plot(dd_hw, dd_pred,ylim = range(y))
lines(inf,col=1)
legend("bottomright",legend=c("time Series", "Holt-Winters & Predicted"),col=1:2,lty=rep(1,2),ncol=1)

###################################################
### mFilter
###################################################
library(mFilter)
y.hp = mFilter(inf,filter="HP") # Hodrick-Prescott filter
y.bk = mFilter(inf,filter="BK") # Baxter-King filter
y.cf = mFilter(inf,filter="CF") # Christiano-Fitzgerald filter
y.bw = mFilter(inf,filter="BW") # Butterworth filter
y.tr = mFilter(inf,filter="TR") # Trigonometric regression filter
plot(inf, main="Inflation & Its Estimated Trend", col=1, ylab="")
lines(y.hp$trend,col=2)
lines(y.bk$trend,col=3)
lines(y.cf$trend,col=4)
lines(y.bw$trend,col=5)
lines(y.tr$trend,col=6)
legend("bottom",legend=c("time Series", "HP","BK","CF","BW","TR"),col=1:6,lty=rep(1,6),ncol=2)


# 3. ARMA ESTIMATION
temp=read.csv("tw_economicGrowth.csv")
growth=ts(temp[,2],start=c(1962,1),freq=4)
plot(growth, ylab="", main="xWg٦v, %")
m.ar2 = arima(growth, order = c(1,0,1))
summary(m.ar2)                       # Print it out

# 4. Automatic model selection
library(forecast)
auto.arima(growth)

# 5. ARMA DIAGNOSTICS
tsdiag(m.ar2)

print(Box.test(m.ar2$residuals, lag=12, type="Ljung-Box")) 	## Time series structure in errors
print(Box.test(m.ar2$residuals^2, lag=12, type="Ljung-Box"))	## Sniff for ARCH
plot(density(m.ar2$residuals), col="blue", xlim=c(-8,8),main=paste("Residuals of AR(2)")) 


### Tests for serial correlation
temp=read.csv("tw_IS.csv")
dataz=ts(temp[,2:3],start=c(1981,1),freq=4)
Invest=dataz[,1]
Save=dataz[,2]
reg1=lm(Invest~Save,data=dataz)
summary(reg1)

library(lmtest)
dwtest(reg1) 
bgtest(reg1,order=4) 
Box.test(reg1$residuals, lag=4, type="Ljung-Box")

### Regression corrected for serial correlation
reg2=arima(Invest, xreg=Save, include.mean = TRUE, order=c(2,0,0))
summary(reg2)
Box.test(residuals(reg2), lag=1, type = "Ljung-Box")



################
###       More
################
# Automatic model selection
library(forecast)
yhat=auto.arima(y,ic="aic")
summary(yhat)
plot(forecast(yhat,h=20))

library(lmtest)
x=data[,2]
reg1=lm(y~x)
dwtest(reg1)
bgtest(reg1,order=4)
Box.test(reg1$residuals, lag=4,type="Ljung-Box")

reg2=arima(y,xreg=x,include.mean = TRUE, order=c(2,0,0),method = c("CSS-ML"))
print(reg2)
Box.test(residuals(reg2),lag=2,type="Ljung-Box")
reg3=auto.arima(y,xreg=x,ic="aic")
reg3




